{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "f6f51d13-60e2-478c-8e11-1dd61e1cc330",
   "metadata": {},
   "source": [
    "# Plotly graph_objectsの設定\n",
    "\n",
    "公開日 <time datetime=\"2025-01-05\">2025-01-05</time>\n",
    "\n",
    "Pythonのインタラクティブなグラフを描画できるライブラリPlotlyにて、`graph_objects`を使ったグラフの設定方法を解説します。\n",
    "\n",
    "Plotlyには、高水準のAPIであるPlotly Expressと呼ばれるものもありますが、\n",
    "この記事では細かい調節ができる`graph_objects`を対象としています。"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "235466ef-8a8c-4f66-b8e0-326e53775b3d",
   "metadata": {},
   "source": [
    "<!-- START MoshimoAffiliateEasyLink -->\n",
    "<script type=\"text/javascript\">\n",
    "(function(b,c,f,g,a,d,e){b.MoshimoAffiliateObject=a;\n",
    "b[a]=b[a]||function(){arguments.currentScript=c.currentScript\n",
    "||c.scripts[c.scripts.length-2];(b[a].q=b[a].q||[]).push(arguments)};\n",
    "c.getElementById(a)||(d=c.createElement(f),d.src=g,\n",
    "d.id=a,e=c.getElementsByTagName(\"body\")[0],e.appendChild(d))})\n",
    "(window,document,\"script\",\"//dn.msmstatic.com/site/cardlink/bundle.js?20220329\",\"msmaflink\");\n",
    "msmaflink({\"n\":\"pandas\\u0026Plotly 2D\\/3Dデータビジュアライゼーション実装ハンドブック\",\"b\":\"秀和システム\",\"t\":\"\",\"d\":\"https:\\/\\/m.media-amazon.com\",\"c_p\":\"\\/images\\/I\",\"p\":[\"\\/51buhQYpkrL._SL500_.jpg\",\"\\/41vnATKu5RL._SL500_.jpg\",\"\\/41vDFPPpE1L._SL500_.jpg\",\"\\/4150EmIK-VL._SL500_.jpg\",\"\\/41-G1+HL-lL._SL500_.jpg\",\"\\/41fwd4F77vL._SL500_.jpg\",\"\\/31LL+GnIWuL._SL500_.jpg\",\"\\/41XR6ul2enL._SL500_.jpg\",\"\\/41RCmqHCXuL._SL500_.jpg\",\"\\/41qtNs0Fe-L._SL500_.jpg\"],\"u\":{\"u\":\"https:\\/\\/www.amazon.co.jp\\/dp\\/479806890X\",\"t\":\"amazon\",\"r_v\":\"\"},\"v\":\"2.1\",\"b_l\":[{\"id\":1,\"u_tx\":\"Amazonで見る\",\"u_bc\":\"#f79256\",\"u_url\":\"https:\\/\\/www.amazon.co.jp\\/dp\\/479806890X\",\"a_id\":4672318,\"p_id\":170,\"pl_id\":27060,\"pc_id\":185,\"s_n\":\"amazon\",\"u_so\":1},{\"id\":2,\"u_tx\":\"楽天市場で見る\",\"u_bc\":\"#f76956\",\"u_url\":\"https:\\/\\/search.rakuten.co.jp\\/search\\/mall\\/pandas%26Plotly%202D%2F3D%E3%83%87%E3%83%BC%E3%82%BF%E3%83%93%E3%82%B8%E3%83%A5%E3%82%A2%E3%83%A9%E3%82%A4%E3%82%BC%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E5%AE%9F%E8%A3%85%E3%83%8F%E3%83%B3%E3%83%89%E3%83%96%E3%83%83%E3%82%AF\\/\",\"a_id\":4672316,\"p_id\":54,\"pl_id\":27059,\"pc_id\":54,\"s_n\":\"rakuten\",\"u_so\":2}],\"eid\":\"IXitR\",\"s\":\"l\"});\n",
    "</script>\n",
    "<div id=\"msmaflink-IXitR\">リンク</div>\n",
    "<!-- MoshimoAffiliateEasyLink END -->"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "cbab4bd3-bb15-40c1-b463-ffb8d4454a64",
   "metadata": {},
   "source": [
    "## graph_objectsの基礎\n",
    "\n",
    "`graph_objects`を使った簡単なグラフ描画の例を以下に示します。"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "555c2de0-bc3f-4eb2-9a59-14473f387b91",
   "metadata": {},
   "outputs": [],
   "source": [
    "import plotly.graph_objects as go\n",
    "\n",
    "fig = go.Figure(\n",
    "    data=go.Bar(x=[1, 2, 3], y=[1, 3, 2]),\n",
    "    layout={\n",
    "        \"title\": \"Graph Title\",\n",
    "        \"xaxis\": {\"title\": \"X AXIS\"},\n",
    "        \"yaxis\": {\"title\": \"Y AXIS\"},\n",
    "    },\n",
    ")\n",
    "\n",
    "fig.show()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "ee0861ef-2dfb-415e-a2b8-b7308777d9eb",
   "metadata": {},
   "source": [
    "上の例を解説します。\n",
    "まず、`import plotly.graph_objects as go`で`graph_objects`をインポートします。\n",
    "次に、`go.Figure()`でグラフを作成します。\n",
    "`go.Figure()`には主に以下の引数があります。\n",
    "\n",
    "- `data`: グラフに表示するデータ\n",
    "- `layout`: グラフの表示を制御\n",
    "\n",
    "`data`引数には、`go.Bar`（棒グラフ）や`go.Scatter`（散布図、折れ線グラフ）などのオブジェクト、またはこれらのオブジェクトをリストやタプルで与えることができます。\n",
    "\n",
    "`layout`引数には、辞書形式や、`go.Layout`オブジェクトを与えることができます。\n",
    "\n",
    "最後に`fig.show()`を実行してグラフを描画します。"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "e7e1dbe9-144d-4457-bffb-254f46a0168c",
   "metadata": {},
   "source": [
    "### グラフの種類\n",
    "\n",
    "Plotlyの主なグラフを以下に示します（`go.Scatter`, `go.Bar`のように利用します）。\n",
    "\n",
    "2次元グラフ\n",
    "\n",
    "- `Scatter`: 散布図、折れ線グラフ\n",
    "- `Bar`: 棒グラフ\n",
    "- `Pie`: 円グラフ\n",
    "- `Heatmap`: ヒートマップ\n",
    "- `Box`: 箱ひげ図\n",
    "- `Violin`: ヴァイオリンプロット\n",
    "- `Histogram`: ヒストグラム\n",
    "\n",
    "3次元グラフ\n",
    "\n",
    "- `Mesh3d`: 3次元の立体表面\n",
    "- `Scatter3d`: 3次元散布図\n",
    "- `Surface`: サーフェスプロット\n",
    "\n",
    "### layoutオプション\n",
    "\n",
    "主なlayoutオプションを以下に示します。\n",
    "\n",
    "- `title` (str): グラフのタイトル\n",
    "- `xaxis`/`yaxis`: x軸、y軸\n",
    "  - `title` (str): x軸、y軸のタイトル\n",
    "  - `range` (tuple): 軸の範囲\n",
    "- `legend`: 凡例\n",
    "  - `title` (str): 凡例のタイトル\n",
    "\n",
    "`title`や`xaxis`/`yaxis`には`font`オプションで以下を指定できます。\n",
    "\n",
    "- `size` (int): フォントサイズ\n",
    "- `color` (str): 文字色"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "d73d38b0-04a9-468b-b35a-12844ef0cc10",
   "metadata": {},
   "source": [
    "## 複数系列のグラフ\n",
    "\n",
    "複数系列のグラフをプロットするには、`go.Figure`の`data`引数にグラフデータをリストやタプルで複数与えます。\n",
    "`name`引数に系列名を与えることができます。"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "7b01b236-5e4a-40d7-b516-de71edc67dc3",
   "metadata": {
    "scrolled": true
   },
   "outputs": [],
   "source": [
    "fig = go.Figure(\n",
    "    data=[\n",
    "        go.Bar(x=[1, 2, 3], y=[1, 3, 2], name=\"Series 1\"),\n",
    "        go.Bar(x=[1, 2, 3], y=[3, 2, 1], name=\"Series 2\"),\n",
    "    ],\n",
    ")\n",
    "fig.show()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "9797aa47-1c5b-468a-b2b9-bfbf5ec56c8f",
   "metadata": {},
   "source": [
    "もしくは`add_trace()`, `add_traces()`メソッドを使用して、`go.Figure`に後からグラフデータを追加することもできます。\n",
    "\n",
    "- `add_trace()`メソッド：グラフデータのインスタンスを1個だけ追加する。\n",
    "- `add_traces()`メソッド：グラフデータのインスタンスをリストやタプルで複数追加する。\n",
    "\n",
    "上記と同じ結果を以下で再現できます。\n",
    "\n",
    "`add_trace()`メソッドの例：\n",
    "\n",
    "```python\n",
    "fig = go.Figure()\n",
    "fig.add_trace(go.Bar(x=[1, 2, 3], y=[1, 3, 2], name=\"Series 1\"))\n",
    "fig.add_trace(go.Bar(x=[1, 2, 3], y=[3, 2, 1], name=\"Series 2\"))\n",
    "fig.show()\n",
    "```\n",
    "\n",
    "`add_traces()`メソッドの例：\n",
    "\n",
    "```python\n",
    "fig = go.Figure()\n",
    "fig.add_traces(\n",
    "    [\n",
    "        go.Bar(x=[1, 2, 3], y=[1, 3, 2], name=\"Series 1\"),\n",
    "        go.Bar(x=[1, 2, 3], y=[3, 2, 1], name=\"Series 2\"),\n",
    "    ]\n",
    ")\n",
    "fig.show()\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "edfb8200-7623-44c0-9a18-fa096c47551f",
   "metadata": {},
   "source": [
    "## レイアウトの設定\n",
    "\n",
    "グラフのレイアウトを変更するには、`go.Figure`の`layout`オプション以外に以下のメソッドで後から変更できます。\n",
    "\n",
    "- `update_layout()`: グラフ全体のレイアウト\n",
    "- `update_xaxes()`: x軸の設定\n",
    "- `update_yaxes()`: y軸の設定\n",
    "\n",
    "各メソッドで指定できるオプションを以下に示します。\n",
    "\n",
    "### update_layout\n",
    "\n",
    "- `showlegend` (bool): `False`にすると凡例を非表示にする\n",
    "- `title` (str): グラフ全体のタイトル\n",
    "- `width` (float): グラフ全体の横幅\n",
    "- `height` (float): グラフ全体の高さ\n",
    "- `hovermode`: `False`にするとカーソルをあてたときにデータを表示しない\n",
    "- `template` (str): 表示スタイルを指定。\n",
    "\n",
    "`template`は以下から指定できます（デフォルトは`plotly`）。\n",
    "\n",
    "```\n",
    "'ggplot2', 'seaborn', 'simple_white', 'plotly', 'plotly_white',\n",
    "'plotly_dark', 'presentation', 'xgridoff', 'ygridoff', 'gridon', 'none'\n",
    "```\n",
    "\n",
    "### update_xaxes/yaxes\n",
    "\n",
    "- `title` (str): 軸のラベル\n",
    "- `range` (tuple, list): 軸の範囲\n",
    "- `rangeslider` (dict): `{\"visible\":True}`とするとレンジスライダーを表示\n",
    "\n",
    "レンジスライダーの例を以下に示します。下側のつまみを左右に動かすことにより、上側のグラフを拡大・縮小できます。"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "1c5efdff-9ea2-4e4f-86e3-ef6f24ad5daf",
   "metadata": {},
   "outputs": [],
   "source": [
    "fig = go.Figure()\n",
    "fig.add_trace(go.Scatter(x=[1, 2, 3], y=[1, 3, 2], name=\"Series 1\"))\n",
    "fig.add_trace(go.Scatter(x=[1, 2, 3], y=[3, 2, 1], name=\"Series 2\"))\n",
    "fig.update_xaxes(rangeslider={\"visible\":True})\n",
    "fig.show()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "d2513684-8b51-4e3b-9dcc-0a413322cf03",
   "metadata": {},
   "source": [
    "## グラフのフォント\n",
    "\n",
    "グラフ中のフォントを変更する例を示します。\n",
    "以下ではフォントサイズを`30`, 文字色を赤としています。"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "5e75b24c-67d5-4611-8a7b-492552d9ca7b",
   "metadata": {},
   "outputs": [],
   "source": [
    "fig = go.Figure(\n",
    "    data=go.Bar(x=[1, 2, 3], y=[1, 3, 2]),\n",
    "    layout={\n",
    "        \"title\": {\n",
    "            \"text\": \"Graph Title\",\n",
    "            \"font\": {\n",
    "                \"size\": 30,\n",
    "                \"color\": \"red\",\n",
    "            },\n",
    "        },\n",
    "    },\n",
    ")\n",
    "fig.show()"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.12.4"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
